import torch
from torchvision import transforms
import torchvision
from pathlib import Path
from models import get_model
from load_zivid_file import load_zdf
from matplotlib import pyplot as plt
import numpy as np
from pose_estimation import segment_pcd, remove_zeros
from open3d import *


input_transform = transforms.Compose([
    transforms.ToPILImage(),
    transforms.Resize((768, 1024)),
    transforms.ToTensor(),
    ])

if __name__ == '__main__':
    """
    Script to evaluate data taken with Zivid camera
    """

    num_classes = 2

    #CHECK IF GPU IS AVAILABLE
    device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
    print("Device found: ", device)

    model = get_model(num_classes=num_classes)

    #Checks if a trained model already exists
    root = './history/binary'
    fold = 'run_iou_fullsize'
    root = Path(root)
    model_path = root / 'model_{fold}.pt'.format(fold=fold)

    if model_path.exists():
        print(" A model exists \n")
        state = torch.load(str(model_path))
        epoch = state['epoch']+1
        batch = state['batch']
        model.load_state_dict(state['model'])
        print(' Restored model, epoch {}, step {:,}\n'.format(epoch, batch))

    model = model.eval()

    #ZIVID DATA
    pcd, rgb, depth = load_zdf('C:/Users/cpascheh/Desktop/workspace/rgb/zivid_data/02062019/pcd1.zdf')
    pcd[np.isnan(pcd[:, :, 2])] = 0
    pcd = pcd[:, :, [0, 1, 2, 4, 5, 6]]

    plt.imshow(rgb)
    plt.title("RGB input image")
    plt.xlabel('Pixels')
    plt.ylabel('Pixels')
    plt.show()

    input_img = input_transform(rgb.data)
    input_img = input_img.unsqueeze(0)
    outputs = model(input_img)

    out_mask = torch.argmax(outputs, dim=1).float()
    plt.imshow(out_mask[0, :, :])
    plt.title("Predicted mask")
    plt.xlabel('Pixels')
    plt.ylabel('Pixels')
    plt.show()

    seg_pcd = segment_pcd(pcd, out_mask)
    seg_pcd = remove_zeros(seg_pcd)

    #For visualizing a point cloud using open3D

    #pts = pcd.data
    #seg = PointCloud()
    #seg.points = Vector3dVector(pts[:, :3])
    #seg.colors = Vector3dVector(pcd[:, :, 3:] / 255)
    #draw_geometries([seg])

