from loader import VisdomLoader
from torchvision import transforms
from torch.utils.data import DataLoader


def calc_mean_and_std(dir, transformation):
    """
    Calculates the mean and std for a given dataset. Used to normalize the dataset -> training is more stable.

    :param dir:
    :param transformation:
    :return: the mean and standard deviation of the desired dataset.
    """

    data_dir = dir
    transform = transformation
    data_ds = VisdomLoader(data_dir, input_transform=transform, target_transform=None)
    dataloader = DataLoader(data_ds, batch_size=1, shuffle=False, num_workers=0)

    mean = 0.
    std = 0.
    nb_samples = 0.
    for data, _ in dataloader:
        batch_samples = data.size(0)
        data = data.view(batch_samples, data.size(1), -1)
        mean += data.mean(2).sum(0)
        std += data.std(2).sum(0)
        nb_samples += batch_samples

    mean /= nb_samples
    std /= nb_samples

    return mean, std


def check_crop_size(image_height, image_width):
    """
    Checks if image size divisible by 32.
    :param image_height:
    :param image_width:
    :return: True if both height and width divisible by 32 and False otherwise.
    """
    return image_height % 32 == 0 and image_width % 32 == 0


if __name__ == '__main__':
    input_transform = transforms.Compose([
        transforms.CenterCrop((768, 1024)),
        transforms.ToTensor()
    ])

    data_dir = './datasets/wisdom/wisdom-real/high-res/'

    transform = input_transform
    data_ds = VisdomLoader(data_dir, input_transform=transform, target_transform=None)
    dataloader = DataLoader(data_ds, batch_size=1, shuffle=False, num_workers=0)
    mean, std = calc_mean_and_std(data_dir, input_transform)