import torch.utils.data as data
import torch
from torchvision import transforms
import torch.nn as nn
from torch.optim import Adam

from train import train_model
from loader import VisdomLoader
from models import get_model
from transform import binarize


input_transform = transforms.Compose([
    transforms.CenterCrop((768, 1024)),
    #transforms.Resize((480, 640)),
    transforms.ToTensor(),
    transforms.Normalize(mean=[0.2092, 0.2172, 0.2155], std=[0.1872, 0.1733, 0.1625]),
    ])

target_transform = transforms.Compose([
    transforms.CenterCrop((768, 1024)),
    #transforms.Resize((480, 640)),
    binarize()
    ])

depth_transform = transforms.Compose([
    transforms.CenterCrop((768, 1024)),
    #transforms.Resize((480, 640)),
    transforms.ToTensor()
    ])


if __name__ == '__main__':

    #MODEL PARAMETERS
    num_channels = 3
    num_classes = 2
    batch_size = 8
    num_epochs = 20
    learning_rate = 1e-4
    train_val_ratio = 0.8

    #the script will make a file where the model and log will be saved. (if the folder doesnt exists)
    root = './history/binary'
    fold = 'testtesttest'

    #Check for GPU
    device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
    print("Device found: ", device)

    #Data loading
    data_dir = './datasets/wisdom/wisdom-real/high-res'
    data_ds = VisdomLoader(data_dir, input_transform=input_transform,
                           target_transform=target_transform,
                           depth_transform=depth_transform)

    train_size = int(train_val_ratio * len(data_ds))
    val_size = len(data_ds) - train_size
    train_dataset, val_dataset = data.random_split(data_ds, [train_size, val_size])
    train_loader = data.DataLoader(train_dataset, batch_size=batch_size, shuffle=False, num_workers=0)
    val_loader = data.DataLoader(val_dataset, batch_size=batch_size, shuffle=False, num_workers=0)

    print('num train = {}, num_val = {}'.format(len(train_dataset), len(val_dataset)))


    #LOAD MODEL, OPTMIZER AND CRITERION
    model = get_model(num_classes=num_classes)
    optimizer = Adam(model.parameters(), lr=learning_rate)
    criterion = nn.CrossEntropyLoss(reduction='sum')



    #TRAIN MODEL
    model, hist, weights, best_model = train_model(model, train_loader, val_loader, criterion, optimizer, device,
                                learning_rate,
                                batch_size,
                                num_classes,
                                root,
                                num_epochs=num_epochs,
                                fold=fold)
