from open3d import *
import numpy as np
import matplotlib.pyplot as plt
import cv2
from visualizer import mask_pc_overlay, visualize_pcd


def point_cloud_generator(color_img, depth_img, matrix):
    """
    Generates a point cloud from rgbd image
    :param image: rgb image
    :param depth: depth image
    :param matrix: camera intrinsic matrix
    :return: point cloud
    """

    rgbd_image = create_rgbd_image_from_color_and_depth(
            color_img, depth_img, convert_rgb_to_intensity=False)

    pc = create_point_cloud_from_rgbd_image(
            rgbd_image, matrix)

    return pc


def read_rgbd_img_and_mask(color_source, depth_source, mask_source, gt_source):
    """
    Reads rgb and depth image and converts them to right format
    :param color_source:
    :param depth_source:
    :return:
    """
    color = read_image(color_source)
    depth = read_image(depth_source)
    mask = read_image(mask_source)
    gt_mask = read_image(gt_source)

    return color, depth, mask, gt_mask


if __name__ == '__main__':
    """
    Script for generating original and segmented point clouds from WISDOM-Real dataset. 
    """

    #Hardcoded parameters from WISDOM-Real dataset
    cam_mat = PinholeCameraIntrinsic(1032, 772, 1105.0, 1105.0, 511.5, 384.0)

    i = 1
    color_source = './results/images/input_color_' + str(i) + '1.png'
    depth_source = './results/images/input_depth_' + str(i) + '1.png'
    mask_source = './results/images/pred_mask_' + str(i) + '.png'
    gt_source = './results/images/gt_mask_' + str(i) +'.png'

    color, depth, mask, gt_mask = read_rgbd_img_and_mask(color_source, depth_source, mask_source, gt_source)
    rgbd_image = create_rgbd_image_from_color_and_depth(color, depth, convert_rgb_to_intensity=False)

    seg_image = mask_pc_overlay(color, mask)


    ## Segment 3D data by segmenting depth image
    depth_arr = np.asarray(depth)
    mask = np.asarray(mask)

    seg_depth = np.where(mask > 0, depth_arr, mask)
    seg_depth = Image(seg_depth.astype(np.uint16))

    # For visualization
    plt.subplot(1, 3, 1)
    plt.title('Color image')
    plt.imshow(rgbd_image.color)
    plt.subplot(1, 3, 2)
    plt.title('Depth image')
    plt.imshow(rgbd_image.depth)
    plt.subplot(1, 3, 3)
    plt.title('seg image')
    plt.imshow(mask)
    plt.show()


    pcd = point_cloud_generator(color, depth, cam_mat)
    visualize_pcd(pcd)
    seg_pcd = point_cloud_generator(seg_image, seg_depth, cam_mat)
    visualize_pcd(seg_pcd)

