import os
import os.path
from PIL import Image
import numpy as np
import matplotlib.pyplot as plt

train_object_dict = ['Background', 'Zoinks', 'Cleaning_flask', 'Banana', 'Noise wipes', 'Hammer', 'Bialetti', 'Fish end',
               'Large Pinch', 'Excel marker', 'Gorilla glue', 'Peach', 'Playing_cards', 'Watermelon', 'Flashlight',
               'Toothbrush', 'Crayons', 'Toothpaste', 'Small_Pinch', 'Spice', 'Corn', 'School glue', 'Button',
               'Egg', 'Mayonnaise']

object_list = ['Lime', 'Dices', 'Sardine box', 'Vaseline', 'red thing', 'Leather pocket', 'Citrus', 'Sunscreen',
               'nutrition bar', 'Face cream', 'pear', 'Dasani drops', 'Plier', 'lighter', 'paper roll', 'Screw pinch',
               'Moore holder', 'Tomcat', 'Noodle soup can', 'Air fresher', 'spray', 'Cili powder', 'cigar', 'pen']

EXTENSIONS = ['.jpg', '.png']


def is_image(filename):
    return any(filename.endswith(ext) for ext in EXTENSIONS)


def image_basename(filename):
    return os.path.basename(os.path.splitext(filename)[0])


def image_path(root, basename, extension):
    return os.path.join(root, f'{basename}{extension}')


if __name__ == '__main__':
    """
    Script for relabeling WISDOM-Real data to check if it could be used for instance segmentation (it could not...)
    """

    n_classes = 25 # 24 object + background
    data_dir = './datasets/wisdom/wisdom-real/high-res'
    masks_root = os.path.join(data_dir, 'modal_segmasks')
    image_root = os.path.join(data_dir, 'color_ims')
    depth_root = os.path.join(data_dir, 'depth_ims')
    filenames = [image_basename(f)
                 for f in os.listdir(masks_root) if is_image(f)]

    test_indices = np.load('c:/Users/cpascheh/Desktop/workspace/rgb/datasets/wisdom/wisdom-real/high-res/test_indices.npy')

    for index in test_indices[114:]:
        print("Image name: ", filenames[index], '\n')

        with open(image_path(image_root, filenames[index], '.png'), 'rb') as f:
            image = Image.open(f).convert('RGB')
            image.save('./datasets/wisdom/wisdom-real/high-res/modified_color_ims/' + filenames[index] + '.png')

        with open(image_path(depth_root, filenames[index], '.png'), 'rb') as f:
            depth = Image.open(f).convert('L')
            depth.save('./datasets/wisdom/wisdom-real/high-res/modified_depth_ims/' + filenames[index] + '.png')

        with open(image_path(masks_root, filenames[index], '.png'), 'rb') as f:
            mask = Image.open(f).convert('L')
            mask = np.asarray(mask)

        #Iterate through the different object in the image and relabeling them
        print("Total objects in image: ", mask.max())
        new_mask = mask.copy()
        for obj in range(1, mask.max()+1):
            print("Objects number in image: ", obj)
            vis_mask = np.where(mask == obj, 1, 0)

            plt.ion()
            plt.imshow(vis_mask)
            plt.show(block=False)
            obj_nr = input("Right pixel value?")
            plt.clf()
            new_mask = np.where(mask == obj, float(obj_nr), new_mask)

        plt.imshow(new_mask)
        plt.show()
        im = Image.fromarray(new_mask).convert("L")
        #im.save('./datasets/wisdom/wisdom-real/high-res/modified_segmasks/' + filenames[index] + '.png')
        print('\n')
