import torch
import numpy as np
import random
import torchvision.transforms.functional as TF


class binarize():

    def __call__(self, image):
        image = np.asarray(image)
        image = np.where(image > 0, 1, 0)

        return torch.from_numpy(image).long().unsqueeze(0)


def colormap(n):
    """
    Generate a 8 bit (0,255) colormap for n classes
    :param n: number of classes
    :return: colormap
    """
    cmap=np.zeros([n, 3]).astype(np.uint8)

    for i in np.arange(n):
        r, g, b = np.zeros(3)

        for j in np.arange(8):
            r = r + (1<<(7-j))*((i&(1<<(3*j))) >> (3*j))
            g = g + (1<<(7-j))*((i&(1<<(3*j+1))) >> (3*j+1))
            b = b + (1<<(7-j))*((i&(1<<(3*j+2))) >> (3*j+2))

        cmap[i,:] = np.array([r, g, b])

    return cmap


def augmentation(image, mask, depth):
    """
    Randomly augments the data (equal augmentation for all), before it goes to the transformation.
    :param image: input image as PIL
    :param mask: mask as PIL
    :param depth: depth as PIl
    :return: The augmented data
    """

    if random.random() > 0.7:
        # random rotation
        angle = random.randint(-90, 90)
        image = TF.rotate(image, angle)
        mask = TF.rotate(mask, angle)
        depth = TF.rotate(depth, angle)

    if random.random() > 0.7:
        # horizontal flip
        image = TF.hflip(image)
        mask = TF.hflip(mask)
        depth = TF.hflip(depth)

    if random.random() > 0.7:
        # Ajust brightness
        brightness = random.uniform(0.5, 2.0)  # reduce/increase with a factor of 0.5/2.0
        image = TF.adjust_brightness(image, brightness)
    return image, mask, depth


def image_to_tensor(image):
    """
    Convert a numpy.ndarray to tensor
    :param image: input image
    :return: image tensor
    """
    #image = image.astype(np.float32)
    image = image.transpose((2, 0, 1))
    tensor = torch.from_numpy(image)

    return tensor


